@@ -1,6 +1,6 @@
 <?php declare(strict_types = 0);
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -16,7 +16,7 @@
 
 ?>
 
-window.widget_host_availability_form = new class {
+window.widget_form = new class extends CWidgetForm {
 
 	init() {
 		for (const element of document.querySelectorAll('[name="interface_type[]"]')) {
@@ -24,6 +24,7 @@ window.widget_host_availability_form = new class {
 		}
 
 		this.#updateForm();
+		this.ready();
 	}
 
 	#updateForm() {
